
# q-expansion of newform 189.4.a.h, downloaded from the LMFDB on 15 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 189
weight = 4
poly_data = [-5, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-1, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, -1], [0, 0], [0, -1], [7, 0], [0, -5], [44, 0], [0, -14], [-49, 0], [0, 17], [0, 48], [191, 0], [29, 0], [0, 81], [386, 0], [0, 14], [0, 28], [0, 154], [128, 0], [-988, 0], [0, 177], [266, 0], [146, 0], [0, -312], [0, -175], [152, 0], [0, 274], [2063, 0], [0, -16], [2171, 0], [0, 258], [-1324, 0], [0, -202], [0, -548], [404, 0], [0, 146], [2738, 0], [-1372, 0], [2792, 0], [0, -90], [0, -523], [0, -164], [-70, 0], [0, -517], [-1042, 0], [0, -534], [3575, 0], [-118, 0], [-511, 0], [0, 1234], [-2140, 0], [0, -1126], [0, 900], [-1510, 0], [0, 600], [0, 1673], [0, -1379], [0, 151], [2504, 0], [-6595, 0], [0, -1290], [1244, 0], [0, 1290], [6659, 0], [0, -212], [-4900, 0], [0, 2030], [68, 0], [-6757, 0], [0, -2081], [-1090, 0], [0, 355], [0, 800], [4487, 0], [-229, 0], [4166, 0], [0, -2542], [0, -1172], [12962, 0], [0, -2176], [-12238, 0], [0, 2370], [-6235, 0], [0, 1615], [-4384, 0], [5432, 0], [0, -403], [0, 3342], [-979, 0], [0, -1491], [-10078, 0], [0, -3106], [0, 3736], [446, 0], [0, 171], [16598, 0], [0, -2376], [0, -766], [0, 355], [18335, 0], [-11947, 0], [6236, 0], [0, -3518], [0, -608], [0, -4414], [12938, 0], [19514, 0], [0, 2634], [0, -5437], [0, -965], [22724, 0], [-3388, 0], [-12643, 0], [0, 2646], [-2455, 0], [-8758, 0], [0, 2158], [-5581, 0], [0, 1450], [0, 2546], [0, 2367], [12488, 0], [-21310, 0], [0, 3245], [0, 3601], [21404, 0], [0, -2664], [15779, 0], [0, 2786], [24848, 0], [34634, 0], [4178, 0], [0, 3081], [950, 0], [-5158, 0], [0, 4394], [21476, 0], [0, 8773], [-22192, 0], [0, 1545], [0, 3064], [-12355, 0], [0, -4738], [-6910, 0], [0, -8859], [-5530, 0], [0, -1728], [-21112, 0], [0, -8579], [31295, 0], [0, 6812], [-106, 0], [0, -4425], [-37816, 0], [0, 10640], [24158, 0], [0, -2868], [22772, 0], [0, -4114], [-49492, 0], [0, -2405], [0, 7133], [0, -6312], [-39502, 0], [0, -7384], [0, 6916], [0, -3854], [14456, 0], [-35170, 0]]
